package ink.wgink.module.wps.util;

import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.wps.consts.IWpsConsts;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import static org.apache.tomcat.util.codec.binary.Base64.encodeBase64String;

/**
 * @ClassName: WpsUtil
 * @Description: wps工具
 * @Author: wanggeng
 * @Date: 2022/6/21 21:58
 * @Version: 1.0
 */
public class WpsUtil {

    private static final Logger LOG = LoggerFactory.getLogger(WpsUtil.class);
    private static final String[] WPS_FILE_TYPE_W = new String[]{"doc", "dot", "wps", "wpt", "docx", "dotx", "docm", "dotm", "rtf"};
    private static final String[] WPS_FILE_TYPE_P = new String[]{"ppt", "pptx", "pptm", "ppsx", "ppsm", "pps", "potx", "potm", "dpt", "dps"};
    private static final String[] WPS_FILE_TYPE_S = new String[]{"xls", "xlt", "et", "xlsx", "xltx", "csv", "xlsm", "xltm"};
    private static final String[] WPS_FILE_TYPE_F = new String[]{"pdf"};
    private static final List<String> WPS_FILE_TYPES = new ArrayList<>();

    static {
        WPS_FILE_TYPES.addAll(Arrays.asList(WPS_FILE_TYPE_W));
        WPS_FILE_TYPES.addAll(Arrays.asList(WPS_FILE_TYPE_P));
        WPS_FILE_TYPES.addAll(Arrays.asList(WPS_FILE_TYPE_S));
        WPS_FILE_TYPES.addAll(Arrays.asList(WPS_FILE_TYPE_F));
    }

    /**
     * 判断是WPS文件
     *
     * @param fileType
     * @return
     */
    public static Boolean isWpsFile(String fileType) {
        for (String wpsFileType : WPS_FILE_TYPES) {
            if (StringUtils.equalsIgnoreCase(wpsFileType, fileType)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取wps文件类型
     *
     * @param fileType
     * @return
     */
    public static String getWpsType(String fileType) {
        for (String w : WPS_FILE_TYPE_W) {
            if (StringUtils.equalsIgnoreCase(w, fileType)) {
                return "w";
            }
        }
        for (String p : WPS_FILE_TYPE_P) {
            if (StringUtils.equalsIgnoreCase(p, fileType)) {
                return "p";
            }
        }
        for (String s : WPS_FILE_TYPE_S) {
            if (StringUtils.equalsIgnoreCase(s, fileType)) {
                return "s";
            }
        }
        for (String f : WPS_FILE_TYPE_F) {
            if (StringUtils.equalsIgnoreCase(f, fileType)) {
                return "f";
            }
        }
        throw new ParamsException("文件格式错误");
    }

    /**
     * 获取路径参数
     *
     * @param params
     * @return
     * @throws UnsupportedEncodingException
     */
    public static String getUrlParam(Map<String, String> params) throws UnsupportedEncodingException {
        StringBuilder builder = new StringBuilder();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            if (builder.length() > 0) {
                builder.append('&');
            }
            builder.append(URLEncoder.encode(entry.getKey(), ISystemConstant.CHARSET_UTF8)).append('=').append(URLEncoder.encode(entry.getValue(), "utf-8"));
        }
        return builder.toString();
    }

    /**
     * 签名
     *
     * @param params
     * @param appSecret
     * @return
     */
    public static String getSignature(Map<String, String> params, String appSecret) {
        List<String> keys = new ArrayList();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            keys.add(entry.getKey());
        }

        // 将所有参数按key的升序排序
        Collections.sort(keys, new Comparator<String>() {
            public int compare(String o1, String o2) {
                return o1.compareTo(o2);
            }
        });

        // 构造签名的源字符串
        StringBuilder contents = new StringBuilder("");
        for (String key : keys) {
            if (key == IWpsConsts.KEY_SIGNATURE) {
                continue;
            }
            contents.append(key + "=").append(params.get(key));
            System.out.println("key:" + key + ",value:" + params.get(key));
        }
        contents.append(IWpsConsts.KEY_SECRET_KEY).append("=").append(appSecret);

        // 进行hmac sha1 签名
        byte[] bytes = hmacSha1(appSecret.getBytes(), contents.toString().getBytes());
        //字符串经过Base64编码
        String sign = encodeBase64String(bytes);
        try {
            sign = URLEncoder.encode(sign, ISystemConstant.CHARSET_UTF8);
        } catch (UnsupportedEncodingException e) {
            LOG.error(e.getMessage(), e);
        }
        LOG.debug("WPS sign: {}", sign);
        return sign;
    }

    /**
     * 加密
     *
     * @param key
     * @param data
     * @return
     */
    public static byte[] hmacSha1(byte[] key, byte[] data) {
        try {
            SecretKeySpec signingKey = new SecretKeySpec(key, "HmacSHA1");
            Mac mac = Mac.getInstance(signingKey.getAlgorithm());
            mac.init(signingKey);
            return mac.doFinal(data);
        } catch (NoSuchAlgorithmException e) {
            LOG.error(e.getMessage(), e);
        } catch (InvalidKeyException e) {
            LOG.error(e.getMessage(), e);
        }
        return null;
    }

}
